'--------------------------------------------------------------------------------------------------------------
'
' File:    mwc256.spin
'
' Author:  Michael Rychlik
'
' License: MIT (See end of file for details)
'
' Revision History:
'
' 1.0      08-01-2012  Initial version.
'
'--------------------------------------------------------------------------------------------------------------
'
' Implementation George Marsaglia's MWC256 Pseudo Random Number Generator (PRNG)
' This has a period of about 2 to the power 8222.
' Be sure to seed it with 256 nice high enropy numbers before use. Either form a real random number generator
' or a simpler but high quality PRNG like JKISS32. The latter is preferable if you want to be able
' to reproduce the sequence from a known seed for testing say.
'
'--------------------------------------------------------------------------------------------------------------

{{
{
/* MWC256 from Usenet posting by G. Marsaglia - Period 2^8222 */
static unsigned int Q[256], c=362436;
unsigned int MWC256(void)
{
    unsigned long long t;
    static unsigned char i = 255;
    t = 809430660ULL * Q[++i] + c;
    c = t >> 32;
    return (Q[i] = t);
}
}}
'--------------------------------------------------------------------------------------------------------------
OBJ
    t : "mwc256_umath"          ' The MWC256 PRNG is implemeted using an unsigned math object
                                ' Things could be optimized by encorporating that code directly into MWC256
                                ' at the cost of clarity.

CON
    magic = 809430660

VAR
    long Q[256]      ' The generators state array
    long c           ' The carry as in "Multiply With Carry" MWC
    byte i           ' An index into the state array
    byte si          ' An index used when seed the state array

' Initialize this object, call before anything else
PUB start
    c  := 362436     ' DO NOT change these magic numbers
    i  := 255
    si := 0

' Set the seed for this generator
' Should be called 256 times with different seed values
PUB seed(qin)
    Q[si++] := qin

' Get the next random value from the generator
PUB random
    t.mult(magic, Q[++i])
    t.dadd(t.high, t.low, 0, c)
    c := t.high
    return (Q[i] := t.low)

'--------------------------------------------------------------------------------------------------------------
'                         TERMS OF USE: MIT License
'--------------------------------------------------------------------------------------------------------------
' Permission is hereby granted, free of charge, to any person obtaining a copy of
' this software and associated documentation files (the "Software"), to deal in
' the Software without restriction, including without limitation the rights to
' use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
' the Software, and to permit persons to whom the Software is furnished to do so,
' subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all
' copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
' FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
' COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
' IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
' CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'--------------------------------------------------------------------------------------------------------------

