'--------------------------------------------------------------------------------------------------------------
'
' File:    mwc256_demo.spin
'
' Author:  Michael Rychlik
'
' License: MIT (See end of file for details)
'
' Revision History:
'
' 1.0      08-01-2012  Initial version.
'
'--------------------------------------------------------------------------------------------------------------
'
' Simple demonstation of seeding and using the MWC256 Pseudo Random Number Generator.
'
' The idea hee is to use RealRandom to get genuienly real random seeds for use in a long period
' Pseudo Random Number Generator (PRNG). In this case the MWC256 generator which has a period of
' approximately 2 to the power 8222. After that we shut down RealRandom thus saving a COG for more useful functionality
' and can get very good random numbers for use in the application.
'
' The MWC256 PRNG requires 256 seed values. Here we get those seed values from the smaller JKISS32 PRNG which is itself
' seeded from RealRandom.
' In a real application MWC256 can be seeded directly from ReadRandom.
'
CON
    _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

    rxpin    = 31
    txpin    = 30
    mode     = 0
    baudrate = 115200

    TRIALS   = 1000000

OBJ
    console  : "FullDuplexSerial"
    rr       : "RealRandom"
    prngj    : "jkiss32"
    prngm    : "mwc256"

PUB start | random, seed, i
    console.start (rxpin, txpin, mode, baudrate)                        ' Start the serial console
    'console.str(string("Pseudo Random numbers from MWC256", 13))

    rr.start                                                            ' Start Real Random, uses a COG
    repeat while rr.random == 0                                         ' Wait for Real Random to warm up

    prngj.start                                                         ' Start the jkiss32 Pseudo Random Number generator
                                                                        ' (Uses default seed values)
    prngj.seed (rr.random, rr.random, rr.random, rr.random, rr.random)  ' Seed the jkiss32 PRNG with real random numbers
    rr.stop                                                             ' Free Real Random COG

    prngm.start                                                         ' Start MWC256 PRNG, still must be seeded though.

    repeat i from 0 to 255                                              ' Genertate 256 seed values for mwc256 from jkiss32 output
        seed := prngj.random                                            ' Get next Pseudo Random Number as seed
        prngm.seed(seed)                                                ' Set seed value to mwc256

    repeat i from 1 to TRIALS
        random := prngm.random                                          ' Get some PRNs from MWC256
        console.hex(random, 8)                                          ' ...and print them
        console.tx($0d)
        console.tx($0a)

'--------------------------------------------------------------------------------------------------------------
'                         TERMS OF USE: MIT License
'--------------------------------------------------------------------------------------------------------------
' Permission is hereby granted, free of charge, to any person obtaining a copy of
' this software and associated documentation files (the "Software"), to deal in
' the Software without restriction, including without limitation the rights to
' use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
' the Software, and to permit persons to whom the Software is furnished to do so,
' subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all
' copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
' FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
' COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
' IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
' CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'--------------------------------------------------------------------------------------------------------------


